import {
  LocalVideoTrack,
  RemoteUser,
  useJoin,
  useLocalCameraTrack,
  useLocalMicrophoneTrack,
  usePublish,
  useRTCClient,
  useRemoteUsers,
  useClientEvent,
} from 'agora-rtc-react'

import React, { createContext, useContext, useEffect } from 'react'
import type { IMicrophoneAudioTrack, ICameraVideoTrack } from 'agora-rtc-sdk-ng'
import { Box, Button, CircularProgress, Typography } from '@mui/material'
import { useTheme, type Theme } from '@mui/material/styles'

interface AgoraContextType {
  localCameraTrack: ICameraVideoTrack | null
  localMicrophoneTrack: IMicrophoneAudioTrack | null
  children: React.ReactNode
}

const AgoraContext = createContext<AgoraContextType | null>(null)

export const AgoraProvider: React.FC<AgoraContextType> = ({
  children,
  localCameraTrack,
  localMicrophoneTrack,
}) => (
  <AgoraContext.Provider value={{ localCameraTrack, localMicrophoneTrack, children }}>
    {children}
  </AgoraContext.Provider>
)

export const useAgoraContext = () => {
  const context = useContext(AgoraContext)
  if (!context) throw new Error('useAgoraContext must be used within an AgoraProvider')
  return context
}

export const AgoraManager = ({ children }: { children: React.ReactNode }) => {
  const theme: Theme = useTheme()
  const agoraEngine = useRTCClient()

  const {
    data: uid,
    error: joinError,
    isConnected: isJoined,
    isLoading: isJoining,
  } = useJoin({
    appid: '3af4648782de46ddbf90005f7a68f206',
    channel: 'test',
    token: null,
    uid: null,
  })
  const { isLoading: isLoadingCam, localCameraTrack } = useLocalCameraTrack()
  const { isLoading: isLoadingMic, localMicrophoneTrack } = useLocalMicrophoneTrack()
  const remoteUsers = useRemoteUsers()

  usePublish([localMicrophoneTrack, localCameraTrack])

  useClientEvent(agoraEngine, 'user-joined', (user) => {
    console.log('The user', user.uid, ' has joined the channel')
  })

  useClientEvent(agoraEngine, 'user-left', (user) => {
    console.log('The user', user.uid, ' has left the channel')
  })

  useClientEvent(agoraEngine, 'user-published', (user, mediaType) => {
    console.log('The user', user.uid, ' has published media in the channel')
  })

  useEffect(() => {
    return () => {
      localCameraTrack?.close()
      localMicrophoneTrack?.close()
    }
  }, [])

  return (
    <Box sx={{ width: '100%', height: '100%', backgroundColor: theme.palette.background.default }}>
      {isJoining ? (
        <Box
          sx={{
            height: '100%',
            width: '100%',
            display: 'flex',
            justifyContent: 'center',
            alignItems: 'center',
          }}
        >
          <Typography
            variant='h3'
            textAlign={'center'}
            sx={{ fontSize: '18px', color: theme.palette.common.white, fontWeight: '600' }}
          >
            Joining the meeting...
          </Typography>
        </Box>
      ) : isLoadingCam || isLoadingMic ? (
        <Box
          sx={{
            height: '100%',
            width: '100%',
            display: 'flex',
            justifyContent: 'center',
            alignItems: 'center',
          }}
        >
          <Box
            sx={{
              display: 'flex',
              flexDirection: 'row',
              justifyContent: 'center',
              alignItems: 'center',
            }}
          >
            <CircularProgress
              size={'md'}
              sx={{
                color: theme.palette.common.white,
                height: '20px',
                width: '20px',
                marginRight: '20px',
              }}
            />
            <Typography
              variant='h3'
              textAlign={'center'}
              sx={{ fontSize: '18px', color: theme.palette.common.white, fontWeight: '600' }}
            >
              Loading devices
            </Typography>
          </Box>
        </Box>
      ) : isJoined ? (
        <AgoraProvider
          localCameraTrack={localCameraTrack}
          localMicrophoneTrack={localMicrophoneTrack}
        >
          {children}
          <div id='videos'>
            {/* Render the local video track */}
            <div className='vid' style={{ height: 300, width: 600 }}>
              <LocalVideoTrack track={localCameraTrack} play={true} />
            </div>
            {/* Render remote users' video and audio tracks */}
            {remoteUsers.map((remoteUser) => (
              <div className='vid' style={{ height: 300, width: 600 }} key={remoteUser.uid}>
                <RemoteUser user={remoteUser} playVideo={true} playAudio={true} />
              </div>
            ))}
          </div>
        </AgoraProvider>
      ) : joinError ? (
        <Box
          sx={{
            height: '100%',
            width: '100%',
            display: 'flex',
            justifyContent: 'center',
            alignItems: 'center',
          }}
        >
          <Box
            sx={{
              backgroundColor: '#262625',
              borderRadius: '20px',
              padding: '20px 40px',
              width: '30%',
              display: 'flex',
              flexDirection: 'column',
              justifyContent: 'center',
              alignItems: 'center',
            }}
          >
            <Typography
              variant='h3'
              textAlign={'center'}
              sx={{
                fontSize: '18px',
                color: theme.palette.common.white,
                fontWeight: '600',
                marginTop: '10px',
              }}
            >
              Unexpected error occurred. Please try reloading the page. 😞
            </Typography>
            <Button
              variant='text'
              sx={{
                color: theme.palette.primary.main,
                borderRadius: '10px',
                padding: '10px 20px',
                marginTop: '10px',
                fontWeight: '600',
              }}
              onClick={() => window.location.reload()}
            >
              Reload
            </Button>
          </Box>
        </Box>
      ) : null}
    </Box>
  )
}

// Export the AgoraManager component as the default export
export default AgoraManager
